/*
GPL v2+
*/
#include <Arduino.h>

#include <FS.h>
#include <SPIFFS.h>

#include <SPI.h>
#include <TFT_eSPI.h>
#include <Free_Fonts.h>
TFT_eSPI tft = TFT_eSPI();

#define BUTTON_0 35
#define BUTTON_1 0
#define TFT_LED 4

// Colors
int TFT_COLOR;
#define BACKGROUND_COLOUR 0x0000

void setup()
{
  Serial.begin(115200);
  Serial.println("GO!");

  pinMode(TFT_LED, OUTPUT);
  digitalWrite(TFT_LED, HIGH);

  Serial.println("Lights on!");

  pinMode(BUTTON_0, INPUT);
  pinMode(BUTTON_1, INPUT);

  tft.init();
  tft.setRotation(1);
  tft.fillScreen(TFT_WHITE);

  Serial.println("TFT inited!");

  tft.begin();
  tft.setRotation(1);
  tft.fillScreen(BACKGROUND_COLOUR); // Clear Screen
}

int val = 0;
float gauss = 0;
float oldgauss;
float zero = 2048;
float gain = ( 3300.0 / 4095.0 ) / 1.4; // (ADC 4095 counts, 3.3V, 1.4mV / Gauss)
int batt;
int oldbatt = 200;
bool units = 0;

void loop()
{
  oldgauss = gauss;
  gauss = (0.8 * gauss + 0.2 * gain * (float)(analogRead(A0) - zero));
  tft.setTextSize(1);
  if (gauss != oldgauss){
    tft.fillRect(0, 0, 240, 80, BACKGROUND_COLOUR);
    tft.setFreeFont(&FreeSansBold24pt7b);
    if (gauss >= 0){
      tft.setCursor(5, 50);
      tft.setTextColor(TFT_RED);
      if (units){
        tft.printf("N %5.1f mT", gauss / 10);
      } else {
        tft.printf("N %5.1f G", gauss);
      }
    } else {
      tft.setCursor(5, 50);
      tft.setTextColor(TFT_BLUE);
      if (units){
        tft.printf("S %5.1f mT", -gauss / 10);
      } else {
        tft.printf("S %5.1f G", -gauss);
      }
    }
  }
  

  if (digitalRead(BUTTON_0) == LOW)
  {
    zero = analogRead(A0);
  }

  if (digitalRead(BUTTON_1) == LOW) { 
    units = ! units;
    while (digitalRead(BUTTON_1) == LOW) delay(100);
  }
 
  // charging 2580 end charging 
  int counts = analogRead(34);
  if (counts > 2170){
    batt = 0.2232 * (float)counts - 413;
  } else {
    batt = 0.5 * (float)counts - 1005;
  }
    
  if (batt > 100 || batt < oldbatt){
      tft.setTextSize(1);
      tft.fillRect(0, 80, 240, 135, BACKGROUND_COLOUR);
      tft.setCursor(180, 120);
      tft.setFreeFont(&FreeSansBold12pt7b);
      if (batt > 170){
        tft.setTextColor(TFT_GREEN);
        tft.print("Full");
      } else if (batt > 115) {
        tft.setTextColor(TFT_YELLOW);
        tft.print("Chrg");
      } else {
        tft.setTextColor((batt < 0)? TFT_RED : TFT_YELLOW);
        batt = max(0, min(batt, 100));
        tft.printf("%i%%", batt);
        oldbatt = batt;
      }
  }
  delay(100);
}
